from typing import Dict, List
import os

from django.contrib.staticfiles.finders import get_finders
import sass


def find_static_paths() -> List[str]:
    """
    Finds all static paths available in this Django project.

    :returns:
        List of paths containing static files.
    """
    found_paths = []
    for finder in get_finders():
        if hasattr(finder, "storages"):
            for appname in finder.storages:
                if hasattr(finder.storages[appname], "location"):
                    abspath = finder.storages[appname].location
                    found_paths.append(abspath)
    return found_paths


def find_static_scss() -> List[str]:
    """
    Finds all static scss files available in this Django project.

    :returns:
        List of paths of static scss files.
    """
    scss_files = []
    for finder in get_finders():
        for path, storage in finder.list([]):
            if path.endswith(".scss"):
                fullpath = finder.find(path)
                scss_files.append(fullpath)
    return scss_files


def compile_sass(inpath: str, outpath: str, output_style: str = None, precision: int = None,
                 source_map: bool = False, include_paths: List[str] = None) -> None:
    """
    Calls sass.compile() within context of Django's known static file paths,
    and writes output CSS and/or sourcemaps to file.

    :param str inpath:
        Path to SCSS file or directory of SCSS files.
    :param str outpath:
        Path to a CSS file or directory in which to write output. The path will
        be created if it does not exist.
    :param str output_style:
        Corresponds to `output_style` from sass package.
    :param int precision:
        Corresponds to `precision` from sass package.
    :param bool source_map:
        If True, write a source map along with the output CSS file.
        Only valid when `inpath` is a file.
    :returns:
        None
    """

    # If include paths are not specified, use Django static paths
    include_paths = include_paths or find_static_paths()

    # Additional sass args that must be figured out.
    sassargs = {}  # type: Dict[str, object]

    # Handle input directories.
    if os.path.isdir(inpath):
        # Assume outpath is also a dir, or make it.
        if not os.path.exists(outpath):
            os.makedirs(outpath)
        if os.path.isdir(outpath):
            sassargs.update({"dirname": (inpath, outpath)})
        else:
            raise NotADirectoryError(
                "Output path must also be a directory when input path is a directory."
            )

    # Handle input files.
    outfile = None
    if os.path.isfile(inpath):
        sassargs.update({"filename": inpath})
        if os.path.isdir(outpath):
            outfile = os.path.join(
                outpath, os.path.basename(inpath.replace(".scss", ".css"))
            )
        else:
            outfile = outpath
        if source_map:
            sassargs.update({"source_map_filename": outfile + ".map"})

    # Compile the sass.
    rval = sass.compile(
        output_style=output_style,
        precision=precision,
        include_paths=include_paths,
        **sassargs,
    )

    # Write output.
    # sass.compile() will return None if used with dirname.
    # If used with filename, it will return a string of file contents.
    if rval and outfile:
        # If we got a css and sourcemap tuple, write the sourcemap.
        if isinstance(rval, tuple):
            map_outfile = outfile + ".map"
            outfile_dir = os.path.dirname(map_outfile)
            if not os.path.exists(outfile_dir):
                os.makedirs(outfile_dir, exist_ok=True)
            file = open(map_outfile, "w", encoding="utf8")
            file.write(rval[1])
            file.close()
            rval = rval[0]

        # Write the outputted css to file.
        outfile_dir = os.path.dirname(outfile)
        if not os.path.exists(outfile_dir):
            os.makedirs(outfile_dir, exist_ok=True)
        file = open(outfile, "w", encoding="utf8")
        file.write(rval)
        file.close()



import os
import sys
import time

from django.core.management.base import BaseCommand
import sass

from django_sass import compile_sass, find_static_scss


class Command(BaseCommand):
    help = "Runs libsass including all paths from STATICFILES_FINDERS."

    def add_arguments(self, parser):
        parser.add_argument(
            "in",
            type=str,
            nargs="+",
            help="An scss file, or directory containing scss files",
        )
        parser.add_argument(
            "out",
            type=str,
            nargs="+",
            help="A file or directory in which to output transpiled css",
        )
        parser.add_argument(
            "-g",
            dest="g",
            action="store_true",
            help="Build a sourcemap. Only applicable if input is a file, not a directory.",
        )
        parser.add_argument(
            "-t",
            type=str,
            dest="t",
            default="expanded",
            help="Output type. One of 'expanded', 'nested', 'compact', 'compressed'",
        )
        parser.add_argument(
            "-p",
            type=int,
            dest="p",
            default=8,
            help="Precision. Defaults to 8",
        )
        parser.add_argument(
            "--watch",
            dest="watch",
            action="store_true",
            default=False,
            help="Watch input path and re-generate css files when scss files are changed.",
        )

    def handle(self, *args, **options) -> None:
        """
        Finds all static paths used by the project, and runs sass
        including those paths.
        """

        # Parse options.
        o_inpath = options["in"][0]
        o_outpath = options["out"][0]
        o_srcmap = options["g"]
        o_precision = options["p"]
        o_style = options["t"]

        # Watch files for changes if specified.
        if options["watch"]:
            try:
                self.stdout.write("Watching...")

                # Track list of files to watch and their modified time.
                watchfiles = {}
                while True:
                    needs_updated = False

                    # Build/update list of ALL scss files in static paths.
                    for fullpath in find_static_scss():
                        prev_mtime = watchfiles.get(fullpath, 0)
                        curr_mtime = os.stat(fullpath).st_mtime
                        if curr_mtime > prev_mtime:
                            needs_updated = True
                            watchfiles.update({fullpath: curr_mtime})

                    # Recompile the sass if needed.
                    if needs_updated:
                        # Catch compile errors to keep the watcher running.
                        try:
                            compile_sass(
                                inpath=o_inpath,
                                outpath=o_outpath,
                                output_style=o_style,
                                precision=o_precision,
                                source_map=o_srcmap,
                            )
                            self.stdout.write("Updated files at %s" % time.time())
                        except sass.CompileError as exc:
                            self.stdout.write(str(exc))

                    # Go back to sleep.
                    time.sleep(3)

            except (KeyboardInterrupt, InterruptedError):
                self.stdout.write("Bye.")
                sys.exit(0)

        # Write css.
        self.stdout.write("Writing css...")
        compile_sass(
            inpath=o_inpath,
            outpath=o_outpath,
            output_style=o_style,
            precision=o_precision,
            source_map=o_srcmap,
        )
        self.stdout.write("Done.")



import os
from setuptools import setup, find_packages


with open(os.path.join(os.path.dirname(__file__), "README.md"), encoding="utf8") as readme:
    README = readme.read()

setup(
    name="django-sass",
    version="1.0.0",
    author="CodeRed LLC",
    author_email="info@coderedcorp.com",
    url="https://github.com/coderedcorp/django-sass",
    description=("The absolute simplest way to use Sass with Django. Pure Python, "
                 "minimal dependencies, and no special configuration required!"),
    long_description=README,
    long_description_content_type="text/markdown",
    license="BSD license",
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        "django",
        "libsass"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Environment :: Web Environment",
    ],
)
